{-|
Module      : Gargantext.API.Routes
Description :
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

{-# LANGUAGE ConstraintKinds      #-}
{-# LANGUAGE ScopedTypeVariables  #-}
{-# LANGUAGE TypeFamilies         #-}
{-# LANGUAGE UndecidableInstances #-}

module Gargantext.API.Routes where

import Database.PostgreSQL.Simple.LargeObjects qualified as PSQL
import Gargantext.API.Admin.Auth.Types (AuthenticatedUser, auth_user_id)
import Gargantext.API.Admin.EnvTypes (Env)
import Gargantext.API.Errors.Types (BackendInternalError)
import Gargantext.API.Node.Types (NewWithForm(..), NewTempWithForm(..))
import Gargantext.API.Prelude (GargServer, GargM, IsGargServer)
import Gargantext.API.Routes.Named.Annuaire qualified as Named
import Gargantext.API.Routes.Named.Corpus qualified as Named
import Gargantext.API.Worker (serveWorkerAPI, serveWorkerAPIm, WorkerAPI)
import Gargantext.Core (Lang)
import Gargantext.Core.Types.Individu (User(..))
import Gargantext.Core.Worker.Jobs.Types qualified as Jobs
import Gargantext.Database.Admin.Types.Node (CorpusId)
import Gargantext.Database.Prelude (mkCmd)
import Gargantext.Prelude
import Servant (Get, FormUrlEncoded, JSON)
import Servant.Server.Generic (AsServerT)

----------------------------------------------------------------------
-- For Tests
type WaitAPI = Get '[JSON] Text

waitAPI ::  Int -> GargServer WaitAPI
waitAPI n = do
  let
    m = (10 :: Int) ^ (6 :: Int)
  _ <- liftBase $ threadDelay ( m * n)
  pure $ "Waited: " <> show n
----------------------------------------

-- addCorpusWithQuery :: User -> Named.AddWithQuery (AsServerT (GargM Env BackendInternalError))
-- addCorpusWithQuery user = Named.AddWithQuery $ \cid -> AsyncJobs $
--   serveJobsAPI AddCorpusQueryJob $ \_jHandle q -> do
--     -- limit <- view $ hasConfig . gc_jobs . jc_max_docs_scrapers
--     -- New.addToCorpusWithQuery user cid q (Just $ fromIntegral limit) jHandle
--     void $ Jobs.sendJob $ Jobs.AddCorpusWithQuery { Jobs._acq_args = q
--                                                   , Jobs._acq_user = user
--                                                   , Jobs._acq_cid = cid }
      {- let log' x = do
        printDebug "addToCorpusWithQuery" x
        liftBase $ log x
      -}

addCorpusWithQuery :: User -> Named.AddWithQuery (AsServerT (GargM Env BackendInternalError))
addCorpusWithQuery user =
  Named.AddWithQuery {
    addWithQueryEp = \cId -> serveWorkerAPI $ \p ->
      Jobs.AddCorpusWithQuery { Jobs._acq_args = p
                              , Jobs._acq_user = user
                              , Jobs._acq_cid = cId }
    }

addCorpusWithForm :: User -> Named.AddWithForm (AsServerT (GargM Env BackendInternalError))
addCorpusWithForm user =
  Named.AddWithForm {
    addWithFormEp = \cId -> serveWorkerAPI $ \p ->
      -- /NOTE(adinapoli)/ Track the initial steps outside 'addToCorpusWithForm', because it's
      -- called in a few places, and the job status might be different between invocations.
      -- markStarted 3 jHandle
      -- New.addToCorpusWithForm user cid i jHandle
      Jobs.AddCorpusFormAsync { Jobs._acf_args = p
                              , Jobs._acf_user = user
                              , Jobs._acf_cid = cId }
    }

-- | Same as 'addCorpusWithForm' but uses temporary file stored in postgres, so that
addWithTempFileApi :: (IsGargServer env err m)
                   => AuthenticatedUser
                   -> CorpusId
                   -> WorkerAPI '[FormUrlEncoded] NewWithForm (AsServerT m)
addWithTempFileApi authenticatedUser cId =
  serveWorkerAPIm $ \(NewWithForm { .. }) -> do
    (PSQL.Oid oId) <- mkCmd (createLargeObject bs)
    let args = NewTempWithForm { _twf_filetype = _wf_filetype
                               , _twf_fileformat = _wf_fileformat
                               , _twf_file_oid = fromIntegral oid
                               , _twf_lang = _wf_lang
                               , _twf_name = _wf_name
                               , _twf_selection = _wf_selection }
    pure $ Jobs.AddCorpusTempFileAsync { _actf_args = args
                                       , _actf_cid = cId
                                       , _actf_user = userId }

  where
    userId = authenticatedUser ^. auth_user_id
    createLargeObject bs c = do
      oId <- PSQL.loCreat c
      loFd <- PSQL.loOpen c oId PSQL.WriteMode
      _ <- PSQL.loWrite c loFd bs
      PSQL.loClose c loFd
      pure oId

addAnnuaireWithForm :: Named.AddAnnuaireWithForm (AsServerT (GargM Env BackendInternalError))
addAnnuaireWithForm =
  Named.AddAnnuaireWithForm {
    addWithFormEp = \aId -> serveWorkerAPI $ \i ->
        Jobs.AddToAnnuaireWithForm { _aawf_annuaire_id = aId
                                   , _aawf_args = i }
    } 
